#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "customentity.h"
#include "effects.h"
#include "decals.h"
#include "trains.h"
#include "saverestore.h"


class CSpriteTrain : public CBaseDelay
{
public:
	void Spawn( void );
	void Precache( void );
	void Activate( void );

	void Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );

	void EXPORT Move( void );
	virtual int		Save( CSave &save );
	virtual int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];

	BOOL		m_activated;

	CBaseEntity *m_pentPath;
	float m_flWait;
	float m_moveDistance;
	float m_targetSpeed;
	float m_flStopTime;
};

LINK_ENTITY_TO_CLASS( env_spritetrain, CSpriteTrain );
TYPEDESCRIPTION	CSpriteTrain::m_SaveData[] = 
{
	DEFINE_FIELD( CSpriteTrain, m_activated, FIELD_BOOLEAN ),
	DEFINE_FIELD( CSpriteTrain, m_flWait, FIELD_FLOAT ),
	DEFINE_FIELD( CSpriteTrain, m_moveDistance, FIELD_FLOAT ),
	DEFINE_FIELD( CSpriteTrain, m_pentPath, FIELD_CLASSPTR ),
	DEFINE_FIELD( CSpriteTrain, m_targetSpeed, FIELD_FLOAT ),
	DEFINE_FIELD( CSpriteTrain, m_flStopTime, FIELD_TIME ),	
};

IMPLEMENT_SAVERESTORE( CSpriteTrain, CBaseDelay );

void CSpriteTrain :: Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{
	if ( pev->spawnflags & SF_TRAIN_WAIT_RETRIGGER )
	{
		// Move toward my target
		pev->spawnflags &= ~SF_TRAIN_WAIT_RETRIGGER;
		SetThink( &CSpriteTrain :: Move );
		Move();
	}
	else
	{
		SetThink( NULL );
		pev->spawnflags |= SF_TRAIN_WAIT_RETRIGGER;
		// Pop back to last target if it's available
		if ( pev->enemy )
			pev->target = pev->enemy->v.targetname;
		pev->nextthink = 0;
		pev->velocity = g_vecZero;
	}
}

void CSpriteTrain :: Activate( void )
{
	// Not yet active, so teleport to first target
	if ( !m_activated )
	{
		m_activated = TRUE;
		m_pentPath = Instance( FIND_ENTITY_BY_TARGETNAME ( NULL, STRING(pev->target)) );
    
		UTIL_SetOrigin	(pev, m_pentPath->v.origin );
		
		if ( FStringNull(pev->targetname) )
		{	// not triggered, so start immediately
			pev->nextthink = gpGlobals->time + 0.1;
			SetThink( Move );
		}
		else
			pev->spawnflags |= SF_TRAIN_WAIT_RETRIGGER;
	}
}

void CSpriteTrain :: Spawn( void )
{
	Precache();
	if (pev->speed == 0)
		pev->speed = 100;
	
	if ( FStringNull(pev->target) )
		ALERT(at_console, "SpriteTrain with no target");
	
	pev->movetype = MOVETYPE_NOCLIP;
	pev->solid = SOLID_NOT;
	pev->effects		= 0;
	pev->frame			= 0;
	pev->rendermode = kRenderGlow;
	pev->renderfx = kRenderFxNoDissipation;
	pev->renderamt = 255;
	

	SET_MODEL( ENT(pev), STRING(pev->model) );
	UTIL_SetOrigin(pev, pev->origin);

	m_activated = FALSE;
}


void CSpriteTrain::Precache( void )
{
	PRECACHE_MODEL( (char *)STRING(pev->model) );
	CBaseDelay::Precache();
}

void CSpriteTrain::Move()
{
	// Not moving on a path, return
	if (!m_pentPath)
		return;

	// Subtract movement from the previous frame
	m_moveDistance -= pev->speed * gpGlobals->frametime;

	// Have we moved enough to reach the target?
	if ( m_moveDistance <= 0 )
	{
		// Fire the passtarget if there is one
		if ( m_pentPath->pev->message )
		{
			FireTargets( STRING(m_pentPath->pev->message), this, this, USE_TOGGLE, 0 );
			if ( FBitSet( m_pentPath->pev->spawnflags, SF_CORNER_FIREONCE ) )
				m_pentPath->pev->message = 0;
		}
		// Time to go to the next target
		m_pentPath = m_pentPath->GetNextTarget();

		// Set up next corner
		if ( !m_pentPath )
		{
			pev->velocity = g_vecZero;
		}
		else 
		{
			if ( m_pentPath->pev->speed != 0 )
				m_targetSpeed = m_pentPath->pev->speed;

			Vector delta = m_pentPath->pev->origin - pev->origin;
			m_moveDistance = delta.Length();
			pev->movedir = delta.Normalize();
			m_flStopTime = gpGlobals->time + m_pentPath->GetDelay();
		}
	}

	pev->nextthink = gpGlobals->time;
}